/* 
    Copyright (c) 2012 Microsoft Corporation. All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code. For the terms of the license, please see the 
    license agreement between you and Microsoft.
  
    To see all Code Samples for Windows Phone, visit http://go.microsoft.com/fwlink/?LinkID=219604 
  
*/

using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Phone.Controls;
using Windows.Foundation;
using Windows.Phone.Speech.Recognition;
using Windows.Phone.Speech.Synthesis;
using Windows.Phone.Speech.VoiceCommands;

// Code behind for the Magic Memo sample code main page. This sample uses simple handlers and helper functions to illustrate the use of 
// speech recognition and synthesis in Window Phone 8. It does not implement MVVM and does not store static strings in resources as a 
// globalized application would.
namespace MagicMemo
{
    using HelperFunctions;

    public partial class MainPage : PhoneApplicationPage
    {
        //
        // Code behind for the Magic Memo sample application Main page
        //
        #region Speech objects

        // Speech synthesizer for prompting and confirming
        private SpeechSynthesizer speechOutput;

        // Speech Recognzier for entering memos using the default dictation grammar
        private SpeechRecognizerUI speechInput;

        // Speech Recognizer for getting user commands. Uses mainPageCommands to add a list grammar
        private SpeechRecognizerUI commandReco;

        // List of commands to listen for on the mainpage
        private List<string> mainPageCommands = new List<string>();

        #endregion

        #region Private fields

        // File in isostorage to store memos
        private string fileName = "savedMemos.txt";

        // Uri of the other application page
        private Uri viewMemosUri = new Uri("/ViewMemos.xaml", UriKind.Relative);

        private int numMemosSaved = 0;

        #endregion

        // Constructor for this page 
        public MainPage()
        {
            InitializeComponent(); // Autogenerated, do not modify
            InitializeSpeech();
            InitializeCommandGrammar();
        }

        #region Event handlers

        // Loaded event handler for this page
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            // Handle the case where this page was launched directly by Voice Command
            if (this.NavigationContext.QueryString != null
                && this.NavigationContext.QueryString.ContainsKey("voiceCommandName")) // Page was launched by Voice Command
            {
                // Note: This page has only one voice command which is to enter a new memo, so there is no need to 
                // check the value of the command name.

                // Get a new memo using speech recognition
                GetNewMemoByVoice();
            }
        }

        // Handler for mic tap
        private void mic_Tap(object sender, GestureEventArgs e)
        {
            // Get a new memo using speech recognition
            GetNewMemoByVoice();
        }

        /// Handler for "Save" button tap
        private void SaveBT_Tap(object sender, GestureEventArgs e)
        {
            // Append the current memo to the iso-storage file of stored memos
            Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text);
            // Reset textbox text to blank
            Memo_TB.Text = "";

            UpdateNumberPhraseList("num", ++numMemosSaved, "MagicMemoEnu");
        }

        /// Handler for Cancel button tap
        private void Cancel_BT_Tap(object sender, GestureEventArgs e)
        {
            // Reset textbox text to blank
            Utilities.SetTextBoxTextSafely(Memo_TB, "");
        }

        /// Handler for View Saved Memos button tap
        private void View_BT_Tap(object sender, GestureEventArgs e)
        {
            // Navigate to the ViewMemos.xaml page
            this.NavigationService.Navigate(viewMemosUri);
        }

        #endregion

        #region Speech helper functions

        // Initializes the synthesizer object and two recognizer objects
        private void InitializeSpeech()
        {
            speechOutput = new SpeechSynthesizer();

            // Simple dictation recognizer for filling in memo text
            speechInput = new SpeechRecognizerUI();

            // Set text to display to the user when recognizing
            speechInput.Settings.ExampleText = "Example: \"Buy roses\"";
            speechInput.Settings.ListenText = "Say your memo";



            // Recognizer for getting commands on this page
            commandReco = new SpeechRecognizerUI();

            // Set text to display to the user when recognizing
            commandReco.Settings.ExampleText = "Button name, \"Retry\", or \"Quit\"";
            commandReco.Settings.ListenText = "Say next command...";

            // Note: Grammars are intialized in the InitCommandGrammars method
        }

        // Initializes the grammar used by the commandReco recognizer object
        private void InitializeCommandGrammar()
        {
            // Recursively add "Content" text of each button to the mainPageCommands string list
            GetButtonContentText(this.LayoutRoot, mainPageCommands);

            // Add these three commands that don't have buttons to the mainPageCommands string array
            mainPageCommands.Add("Try again");
            mainPageCommands.Add("Quit");
            mainPageCommands.Add("Save and try again");

            // Add grammar consisting of button "Content" text
            commandReco.Recognizer.Grammars.AddGrammarFromList("mainPageCommands", mainPageCommands);
        }

        // Gets a new memo using recognition. Used by mic_Tap handler and some command results
        // in the CommandCompleted callback
        private async void GetNewMemoByVoice()
        {
            try
            {
                // Prompt the user using speech synthesis
                await speechOutput.SpeakTextAsync("Say your memo");

                // Use the default dictation grammar to get the new memo text
                var recoResult = await speechInput.RecognizeWithUIAsync();

                if (recoResult.ResultStatus == SpeechRecognitionUIStatus.Succeeded)
                {
                    // Show new memo text
                    Utilities.SetTextBoxTextSafely(Memo_TB, recoResult.RecognitionResult.Text);

                    // Ask the user what to do next and execute the command
                    GetCommand();
                }
                else
                {
                    this.Dispatcher.BeginInvoke(delegate { MessageBox.Show(String.Format("Failure message: {0}", recoResult.ResultStatus.ToString())); });
                }
            }
            catch (Exception ex)
            {
                const int privacyPolicyHResult = unchecked((int)0x80045509);

                if (ex.HResult == privacyPolicyHResult) // User has not accepted the speech privacy policy
                {
                    this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("The user has not accepted the speech privacy policy."); });
                }
                else
                {
                    this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("Exception in dictation recognition" + ex.Message); });
                }
            }
        }

        // Gets a command using the commandReco recognition object after the user has entered a memo by Speech recognition. 
        // The result is handled in the CommandCompleted callback.
        private async void GetCommand()
        {
            // Prompt the user to say a command
            await speechOutput.SpeakTextAsync("Say a button name, quit, 'save and try again' or 'try again'.");

            var operation = commandReco.RecognizeWithUIAsync();

            // Use a callback function instead of the await keyword in this case
            operation.Completed = CommandCompleted;
        }

        // Callback for RecognizeWithUIAsync completion for use with the commandReco recognizer object. It takes the appropriate action based on 
        // the recognition result, which can be any button name or "save and try again", "try again", or "quit". Referenced in the "GetCommand" 
        // helper function
        private void CommandCompleted(IAsyncOperation<SpeechRecognitionUIResult> asyncInfo, AsyncStatus asyncStatus)
        {
            // Don't do anything if the recognition hasn't completed.
            if (asyncStatus != AsyncStatus.Completed)
            {
                return;
            }

            try
            {
                // Get the recognition result object
                SpeechRecognitionUIResult result = asyncInfo.GetResults();

                if (result.ResultStatus != SpeechRecognitionUIStatus.Succeeded) // 
                {

                    // Return if not successful. This is a shortcut. A real application would take appropriate action depending 
                    // on the status
                    this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("An error occurred in speech recognition."); });
                    return;
                }

                // Recognition successful -- take the action specified by the user
                switch (result.RecognitionResult.Text.ToLower())
                {
                    // Save this memo and get another one
                    case "save and try again":

                        Memo_TB.Dispatcher.BeginInvoke(() => Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text));

                        Utilities.SetTextBoxTextSafely(Memo_TB, "");

                        numMemosSaved++;

                        GetNewMemoByVoice();

                        break;

                    // Save memo and go back to screen
                    case "save":

                        Memo_TB.Dispatcher.BeginInvoke(() => Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text));

                        numMemosSaved++;

                        Utilities.SetTextBoxTextSafely(Memo_TB, "");

                        break;

                    // Erase this memo and try again
                    case "try again":

                        Utilities.SetTextBoxTextSafely(Memo_TB, "");

                        GetNewMemoByVoice();

                        break;

                    // Erase this memo and go back to screen
                    case "cancel":

                        Utilities.SetTextBoxTextSafely(Memo_TB, "");

                        break;

                    // Save and go to view memos page
                    case "view saved memos":

                        Memo_TB.Dispatcher.BeginInvoke(() => Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text));

                        numMemosSaved++;

                        Utilities.SetTextBoxTextSafely(Memo_TB, "");

                        try
                        {
                            Dispatcher.BeginInvoke(delegate { NavigationService.Navigate(viewMemosUri); });
                        }
                        catch (Exception ex)
                        {
                            this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("Exception in navigating to 'view memos' page" + ex.Message); });
                        }
                        break;

                    // Leave this memo and go back to screen
                    case "quit":

                        break;

                    default:
                        break;
                }

            }
            catch (Exception ex)
            {
                this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("Exception in command recognition handler " + ex.Message); });
            }
        }

        // Updates the "num" PhraseList to have the same number of entries as the number of saved memos.
        // This supports "Magic Memo show memo 5" if there are five or more memos saved.
        private async void UpdateNumberPhraseList(string phraseList, int newLimit, string voiceSetName)
        {
            // Helper function that sets string array to {"1", "2", etc}
            List<string> positiveIntegers = Utilities.GetStringListOfPositiveIntegers(Math.Max(1, newLimit));

            try
            {
                VoiceCommandSet vcs = null;

                if (VoiceCommandService.InstalledCommandSets.TryGetValue(voiceSetName, out vcs))
                {
                    // Update phrase list to the new numbers
                    await vcs.UpdatePhraseListAsync(phraseList, positiveIntegers);
                }
            }
            catch (Exception ex)
            {
                this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("Exception in UpdateNumberPhraseList " + ex.Message); });
            }
        }

        // Recursively crawls through the tree of controls to find all button controls and adds the Content text
        // to a List.
        public static void GetButtonContentText(UIElement uiElement, IList<string> buttonNames)
        {
            Button button = uiElement as Button;
            if (button != null && button.Content != null && !String.IsNullOrEmpty(button.Content.ToString()))
            {
                buttonNames.Add(button.Content.ToString()); // TODO: check to make sure the Content is not a non-text object
            }

            Panel panel = uiElement as Panel;

            if (panel != null)
            {
                foreach (UIElement child in panel.Children)
                {
                    GetButtonContentText(child, buttonNames);
                }
            }
        }
        #endregion
    }
}

