﻿/* 
    Copyright (c) 2012 Microsoft Corporation. All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code. For the terms of the license, please see the 
    license agreement between you and Microsoft.
  
    To see all Code Samples for Windows Phone, visit http://go.microsoft.com/fwlink/?LinkID=219604 
  
*/

//#define SemanticsDebug
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Phone.Controls;
using Windows.Phone.Speech.Recognition;
using Windows.Phone.Speech.VoiceCommands;

// Code behind for the Magic Memo sample code ViewMemos page. This sample uses simple handlers and helper functions to illustrate the use of speech recognition and synthesis
// in Window Phone 8. It does not implement MVVM and does not store static strings in resources as a globalized application would.
namespace MagicMemo
{
    using HelperFunctions;

    public partial class ViewMemos : PhoneApplicationPage
    {
        //
        // Code behind for View Memos page for the Magic Memo sample application View Memos page
        //

        #region Speech objects

        // Recognizer for commanding the page. Uses an SRGS grammar.
        private SpeechRecognizerUI commandReco;

        #endregion

        #region Private fields

        // Observable collection for storing the current set of memos
        private ObservableCollection<string> memoList = new ObservableCollection<string>();

        // File in isostorage to store memos
        private string fileName = "savedMemos.txt";

        #endregion

        // Constructor for this page
        public ViewMemos()
        {
            InitializeComponent(); // Autogenerated
            InitializeSpeech();
        }

        #region EventHandlers

        // Handler for Loaded event.
        private void ViewMemosPage_Loaded(object sender, RoutedEventArgs e)
        {
            // Read saved memos from iso-storage.
            Utilities.ReadLinesFromIsoStorageFile(fileName, memoList); // In HelperFunctions.cs

            // Set the listbox source to the collection of memos loaded from iso-storage.
            MemosLB.ItemsSource = memoList;

            UpdateNumberPhraseList("num", memoList.Count, "MagicMemoEnu");

            // Handle the case where page was launched directly by Voice Command
            if (this.NavigationContext.QueryString != null
                && this.NavigationContext.QueryString.ContainsKey("voiceCommandName"))
            {
                // If in this block then page was launched by Voice Command

                // Take appropriate action depending on the voice command
                string commandName = NavigationContext.QueryString["voiceCommandName"];
                string spokenNumber = "";

                if (commandName == "showOne" && this.NavigationContext.QueryString.TryGetValue("num", out spokenNumber))
                {
                    // Command was "Show memo number 'num'".
                    int index = -1;

                    if (int.TryParse(spokenNumber, out index) && index <= memoList.Count && index > 0)
                    {   // Display the specified memo
                        this.Dispatcher.BeginInvoke(delegate { MessageBox.Show(String.Format("Memo {0}:  \"{1}\"", index, memoList[index - 1])); });
                    }
                    else
                    {
                        // This shouldn't happen after first run unless there are no saved memos, because UpdateNumberPhraseList will have set the 
                        // 'num' PhraseList to be equal to the number of saved memos.
                        this.Dispatcher.BeginInvoke(delegate { MessageBox.Show(String.Format("No memo {0} yet", index + 1)); });
                    }
                }
                // Note: 
                // There is no need for an "else" block in this application because if launched by another VoiceCommand 
                // then commandName="showAll" so the only action is to display the page. An application that has many voice commands 
                // associated with a page could use something like a switch block on the commandName.
            }
        }

        // Handler for 'Clear all' button tap
        private void ClearAllBT_Tap(object sender, GestureEventArgs e)
        {
            // Remove all memos
            memoList.Clear();
        }

        // Handler for 'Clear one' button tap
        private void ClearOneBT_Tap(object sender, GestureEventArgs e)
        {
            // Remove the selected memo 
            if (MemosLB.SelectedIndex > -1)
            {
                memoList.RemoveAt(MemosLB.SelectedIndex);
            }
        }

        // Handler for 'Save' button tap
        private void SaveBT_Tap(object sender, GestureEventArgs e)
        {
            // Save changes to the iso-storage file.
            Utilities.SaveCollectionToIsoStorage(fileName, memoList);

            // Update the num phrase list to match the current number of saved memos.
            UpdateNumberPhraseList("num", memoList.Count, "MagicMemoEnu");
        }

        // Handler for ListBox 'Selection changed'
        private void MemosLB_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Display the number of the selected memo in the TextBox or blank if no selection
            if (MemosLB.SelectedIndex < 0)
            {
                Utilities.SetTextBoxTextSafely(SelectedIndexTB, "");
            }
            else
            {
                Utilities.SetTextBoxTextSafely(SelectedIndexTB, String.Format("{0}", MemosLB.SelectedIndex + 1));
            }
        }

        // Handler for 'Enter New' button tap
        private void EnterNewBT_Tap(object sender, RoutedEventArgs e)
        {
            // Go to main page to enter a new memo
            NavigationService.Navigate(new Uri("/MainPage.xaml", UriKind.Relative));
        }

        // Handler for mic tap
        // Launches a speech recognition session using the commandReco object, which already has the SRGS grammar added
        // and pre-loaded (see InitializeSpeech helper function). This case uses the 'await' construct to get the recognition
        // result rather than an explicit callback function, and executes commands based on the result.
        private async void micImage_Tap(object sender, GestureEventArgs e)
        {
            var commandResult = await commandReco.RecognizeWithUIAsync();

            if (commandResult.ResultStatus == SpeechRecognitionUIStatus.Succeeded
                && commandResult.RecognitionResult != null
                && commandResult.RecognitionResult.Semantics != null)
            {
                // Execute commands based on the semantic values returned rather than the actual utterance recognized. Using
                // semantics has the advantage that there's no need to parse the actual utterance to discern what the user 
                // intended. The code acts on what the user intended, not what he/she actually said.
                // The semantic values are defined in the SRGS grammar <tag> elements using a subset of ECMAscript. see
                // http://www.w3.org/TR/semantic-interpretation/ for more information.
                var commandSemantics = commandResult.RecognitionResult.Semantics;

#if SemanticsDebug // Displays the semantics returned from the grammar for debugging.
                string kv = "Semantics: ";

                foreach (string k in commandSemantics.Keys)
                {
                    kv += String.Format(" {0} = {1}; ", k, commandSemantics[k].Value as string);
                }

               this.Dispatcher.BeginInvoke(delegate { MessageBox.Show(kv); });
#endif
                SemanticProperty process = null;

                if (commandSemantics.TryGetValue("process", out process))
                {
                    SemanticProperty numSemantic = null;
                    SemanticProperty scopeSemantic = null;
                    int selectedIndex = 0;

                    // In general a semantic value can be any object, but in this case it's a string
                    switch (process.Value as string)
                    {
                        // For this grammar the 'process' semantic more or less corresponds to a button on the page.
                        case "select":

                            // The command was "Select memo number 'num'".
                            if (commandSemantics.TryGetValue("num", out numSemantic)
                                && int.TryParse(numSemantic.Value as string, out selectedIndex)
                                && selectedIndex > 0
                                && selectedIndex <= memoList.Count)
                            {
                                MemosLB.SelectedIndex = selectedIndex - 1;
                            }

                            break;

                        case "clear":
                            // Command was "Clear memo number 'num'", "Clear all" or "Clear Selected".

                            if (commandSemantics.TryGetValue("num", out numSemantic))
                            {
                                // The command was "Clear memo number 'num'".
                                if (int.TryParse(numSemantic.Value as String, out selectedIndex))
                                {
                                    selectedIndex--;

                                    if (selectedIndex < memoList.Count && selectedIndex >= 0)
                                    {
                                        memoList.RemoveAt(selectedIndex);
                                    }
                                }
                            }
                            else if (commandSemantics.TryGetValue("scope", out scopeSemantic))
                            {
                                if (scopeSemantic.Value as string == "selected")
                                {
                                    // The command was "Clear selected"
                                    if (MemosLB.SelectedIndex > -1)
                                    {
                                        memoList.RemoveAt(MemosLB.SelectedIndex);
                                    }
                                }
                                else if (scopeSemantic.Value as string == "all")
                                {
                                    // The command was "Clear all"
                                    memoList.Clear();
                                }
                            }

                            break;

                        case "save":
                            // Command was "Save" or "Save Changes"

                            // Save current memo collection
                            Utilities.SaveCollectionToIsoStorage(fileName, memoList);
                            UpdateNumberPhraseList("num", memoList.Count, "MagicMemoEnu");

                            break;

                        case "new":
                            // Command was "Enter new" 

                            // Navigate to MainPage to enter a new memo
                            NavigationService.Navigate(new Uri("/MainPage.xaml", UriKind.Relative));

                            break;

                        default:
                            break;
                    }
                }
            }
        }

        // Display the memoList index on the page, referenced from a textblock in ViewMemos.xaml. 
        // This is kind of a hack. Could possibly use a Binding Converter instead.
        private void TextBlock_Loaded(object sender, RoutedEventArgs e)
        {
            TextBlock tb = sender as TextBlock;
            string value = tb.DataContext as string;
            tb.Text = (memoList.IndexOf(value) + 1).ToString();
        }
        #endregion

        #region Private speech functions
        // Sets up the commandReco speech recognition object and preloads the grammar
        private async void InitializeSpeech()
        {
            if (commandReco == null)
            {
                try
                {
                    // Instantiate speech recognizer
                    commandReco = new SpeechRecognizerUI();

                    // Set prompt spoken to user
                    commandReco.Settings.ListenText = "Say selection or button name";

                    // Set text displayed as examples
                    commandReco.Settings.ExampleText = "Ex. 'Clear three' or 'Select two'";

                    // Path to SRGS grammar to recognize button names and numbers
                    Uri grammarFileUri = new Uri("ms-appx:///ViewMemos.grxml");

                    // Adds grammar but does not load it
                    commandReco.Recognizer.Grammars.AddGrammarFromUri("srgsCommands", grammarFileUri);

                    // Preloads the grammar to reduce latency during recognition. This step is optional. If not done the call to 
                    // RecognizeAsyncUI will do it but recognition may take more time.
                    await commandReco.Recognizer.PreloadGrammarsAsync();
                }
                catch (Exception ex)
                {
                    this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("Exception in commandReco initialization" + ex.Message); });
                }
            }
        }


        // Updates the "num" PhraseList to have the same number of entries as the number of saved memos.
        // This supports "Magic Memo show memo five" if there are five or more memos saved.
        private async void UpdateNumberPhraseList(string phraseList, int newLimit, string commandSetName)
        {
            List<string> positiveIntegers = Utilities.GetStringListOfPositiveIntegers(Math.Max(1, newLimit));

            try
            {
                VoiceCommandSet vcs = null;

                if (VoiceCommandService.InstalledCommandSets.TryGetValue(commandSetName, out vcs))
                {
                    // Update phrase list to the new numbers
                    await vcs.UpdatePhraseListAsync(phraseList, positiveIntegers);
                }
            }
            catch (Exception ex)
            {
                this.Dispatcher.BeginInvoke(delegate { MessageBox.Show("Exception in UpdateNumberPhraseList " + ex.Message); });
            }
        }

        #endregion
    }
}